#!/usr/bin/perl

#
#  get.cgi
#  gets weatherdata from own database - and show it as HTML page
#  for a Virtual HTML Input
#
#  Part of Loxone Weather for Loxberry
#
#  Version 1.0.1
#  Latest change: 01/06/16 05:33:00 Mo
#
#  (C) 2016 by Michael Schlenstedt
#

# Version of this script
my $version = "1.0.1";

# Modules
use DateTime;

# Read settings
require "./data/settings.dat";

# Remove trailing slashes from paths and URLs
our $datadir =~ s/(.*)\/$/$1/eg;

# Date Reference: Convert into Loxone Epoche (1.1.2009)
my $dateref = DateTime->new(
      year      => 2009,
      month     => 1,
      day       => 1,
      time_zone => 'local',
);

# Start Output
print "Content-type: text/plain\n\n";

#
# Print out current conditions
#

# Read data
open(F,"<$datadir/$currentdb") || die "Cannot open $datadir/$currentdb";
  flock(F,2) if($flock);
  our $curdata = <F>;
  flock(F,8) if($flock);
close(F);

chomp $curdata;

my @fields = split(/\|/,$curdata);
our $value;
our $name;

# Check for empty data
if (@fields[0] eq "") {
  @fields[0] = 1230764400;
  @fields[34] = 0;
  @fields[35] = 0;
  @fields[36] = 0;
  @fields[37] = 0;
}

# Correct Epoch by Timezone
$tzseconds = (@fields[4] / 100 * 3600);

# EpochDate - Corrected by TZ
our $epochdate = DateTime->from_epoch(
      epoch      => @fields[0],
);
$epochdate->add( seconds => $tzseconds );

$name = "cur_date";
$value = @fields[0] - $dateref->epoch() + (@fields[4] / 100 * 3600);
&send;

$name = "cur_date_des";
$value = @fields[1];
#&send;

$name = "cur_date_tz_des_sh";
$value = @fields[2];
#&send;

$name = "cur_date_tz_des";
$value = @fields[3];
#&send;

$name = "cur_date_tz";
$value = @fields[4];
#&send;

$name = "cur_day";
$value = $epochdate->day;
&send;

$name = "cur_month";
$value = $epochdate->month;
&send;

$name = "cur_year";
$value = $epochdate->year;
&send;

$name = "cur_hour";
$value = $epochdate->hour;
&send;

$name = "cur_min";
$value = $epochdate->minute;
&send;

$name = "cur_loc_n";
$value = @fields[5];
#&send;

$name = "cur_loc_c";
$value = @fields[6];
#&send;

$name = "cur_loc_ccode";
$value = @fields[7];
#&send;

$name = "cur_loc_lat";
$value = @fields[8];
&send;

$name = "cur_loc_long";
$value = @fields[9];
&send;

$name = "cur_loc_el";
$value = @fields[10];
&send;

$name = "cur_tt";
if (!$metric) {$value = @fields[11]*1.8+32} else {$value = @fields[11]};
&send;

$name = "cur_tt_fl";
if (!$metric) {$value = @fields[12]*1.8+32} else {$value = @fields[12]};
&send;

$name = "cur_hu";
$value = @fields[13];
&send;

$name = "cur_w_dirdes";
$value = @fields[14];
#&send;

$name = "cur_w_dir";
$value = @fields[15];
&send;

$name = "cur_w_sp";
if (!$metric) {$value = @fields[16]*0.621371192} else {$value = @fields[16]};
&send;

$name = "cur_w_gu";
if (!$metric) {$value = @fields[17]*0.621371192} else {$value = @fields[17]};
&send;

$name = "cur_w_ch";
if (!$metric) {$value = @fields[18]*1.8+32} else {$value = @fields[18]};
&send;

$name = "cur_pr";
if (!$metric) {$value = @fields[19]*0.0295301} else {$value = @fields[19]};
&send;

$name = "cur_dp";
if (!$metric) {$value = @fields[20]*1.8+32} else {$value = @fields[20]};
&send;

$name = "cur_vis";
if (!$metric) {$value = @fields[21]*0.621371192} else {$value = @fields[21]};
&send;

$name = "cur_sr";
$value = @fields[22];
&send;

$name = "cur_hi";
if (!$metric) {$value = @fields[23]*1.8+32} else {$value = @fields[23]};
&send;

$name = "cur_uvi";
$value = @fields[24];
&send;

$name = "cur_prec_today"; 
if (!$metric) {$value = @fields[25]*0.0393700787} else {$value = @fields[25]};
&send;

$name = "cur_prec_1hr"; 
if (!$metric) {$value = @fields[26]*0.0393700787} else {$value = @fields[26]};
&send;

$name = "cur_we_icon"; 
$value = @fields[27];
#&send;

$name = "cur_we_code"; 
$value = @fields[28];
&send;

$name = "cur_we_des"; 
$value = @fields[29];
#&send;

$name = "cur_moon_p"; 
$value = @fields[30];
&send;

$name = "cur_moon_a"; 
$value = @fields[31];
&send;

$name = "cur_moon_ph"; 
$value = @fields[32];
#&send;

$name = "cur_moon_h"; 
$value = @fields[33];
#&send;

# Create Sunset/rise Date in Loxone Epoch Format (1.1.2009)
# Sunrise
my $sunrdate = DateTime->new(
      year      => $epochdate -> year(),
      month     => $epochdate -> month(),
      day       => $epochdate -> day(),
      hour      => @fields[34],
      minute    => @fields[35],
);

# Sunset
my $sunsdate = DateTime->new(
      year      => $epochdate -> year(),
      month     => $epochdate -> month(),
      day       => $epochdate -> day(),
      hour      => @fields[36],
      minute    => @fields[37],
);

$name = "cur_sun_r";
$value = $sunrdate->epoch() - $dateref->epoch();
&send;

$name = "cur_sun_s";
$value = $sunsdate->epoch() - $dateref->epoch();
&send;

#
# Print out Daily Forecast
#

# Read data
open(F,"<$datadir/$dailyforecastdb") || die "Cannot open $datadir/$dailyforecastdb";
  flock(F,2) if($flock);
  our @dfcdata = <F>;
  flock(F,8) if($flock);
close(F);

foreach (@dfcdata){
  s/[\n\r]//g;
  my @fields = split(/\|/);

  my $per = @fields[0] - 1;

  # Send values only if we should do so
  my $send = 0;
  foreach (split(/,/,$senddfc)){
    if ($_ eq $per) {
      $send = 1;
    }
  }
  if (!$send) {
    next;
  }

  # Check for empty data
  if (@fields[1] eq "") {
    @fields[1] = 1230764400;
  }

  $name = "dfc$per\_per";
  $value = @fields[0] - 1;
  &send;

  $name = "dfc$per\_date";
  $value = @fields[1] - $dateref->epoch();
  &send;

  $name = "dfc$per\_day";
  $value = @fields[2];
  &send;

  $name = "dfc$per\_month";
  $value = @fields[3];
  &send;

  $name = "dfc$per\_monthn";
  $value = @fields[4];
#  &send;

  $name = "dfc$per\_monthn_sh";
  $value = @fields[5];
#  &send;

  $name = "dfc$per\_year";
  $value = @fields[6];
  &send;

  $name = "dfc$per\_hour";
  $value = @fields[7];
  &send;

  $name = "dfc$per\_min";
  $value = @fields[8];
  &send;

  $name = "dfc$per\_wday";
  $value = @fields[9];
#  &send;

  $name = "dfc$per\_wday_sh";
  $value = @fields[10];
#  &send;

  $name = "dfc$per\_tt_h";
  if (!$metric) {$value = @fields[11]*1.8+32} else {$value = @fields[11];}
  &send;

  $name = "dfc$per\_tt_l";
  if (!$metric) {$value = @fields[12]*1.8+32} else {$value = @fields[12];}
  &send;

  $name = "dfc$per\_pop";
  $value = @fields[13];
  &send;

  $name = "dfc$per\_prec";
  if (!$metric) {$value = @fields[14]*0.0393700787} else {$value = @fields[14];}
  &send;

  $name = "dfc$per\_snow";
  if (!$metric) {$value = @fields[15]*0.393700787} else {$value = @fields[15];}
  &send;

  $name = "dfc$per\_w_sp_h";
  if (!$metric) {$value = @fields[16]*0.621} else {$value = @fields[16];}
  &send;

  $name = "dfc$per\_w_dirdes_h";
  $value = @fields[17];
#  &send;

  $name = "dfc$per\_w_dir_h";
  $value = @fields[18];
  &send;

  $name = "dfc$per\_w_sp_a";
  if (!$metric) {$value = @fields[19]*0.621} else {$value = @fields[19];}
  &send;

  $name = "dfc$per\_w_dirdes_a";
  $value = @fields[20];
#  &send;

  $name = "dfc$per\_w_dir_a";
  $value = @fields[21];
  &send;

  $name = "dfc$per\_hu_a";
  $value = @fields[22];
  &send;

  $name = "dfc$per\_hu_h";
  $value = @fields[23];
  &send;

  $name = "dfc$per\_hu_l";
  $value = @fields[24];
  &send;

  $name = "dfc$per\_we_icon";
  $value = @fields[25];
#  &send;

  $name = "dfc$per\_we_code";
  $value = @fields[26];
  &send;

  $name = "dfc$per\_we_des";
  $value = @fields[27];
#  &send;

}

#
# Print out Hourly Forecast
#

# Read data
open(F,"<$datadir/$hourlyforecastdb") || die "Cannot open $datadir/$hourlyforecastdb";
  flock(F,2) if($flock);
  our @hfcdata = <F>;
  flock(F,8) if($flock);
close(F);

foreach (@hfcdata){
  s/[\n\r]//g;
  my @fields = split(/\|/);

  my $per = @fields[0];

  # Send values only if we should do so
  my $send = 0;
  foreach (split(/,/,$sendhfc)){
    if ($_ eq $per) {
      $send = 1;
    }
  }
  if (!$send) {
    next;
  }

  # Check for empty data
  if (@fields[1] eq "") {
    @fields[1] = 1230764400;
  }

  $name = "hfc$per\_per";
  $value = @fields[0];
  &send;

  $name = "hfc$per\_date";
  $value = @fields[1] - $dateref->epoch();
  &send;

  $name = "hfc$per\_day";
  $value = @fields[2];
  &send;

  $name = "hfc$per\_month";
  $value = @fields[3];
  &send;

  $name = "hfc$per\_monthn";
  $value = @fields[4];
#  &send;

  $name = "hfc$per\_monthn_sh";
  $value = @fields[5];
#  &send;

  $name = "hfc$per\_year";
  $value = @fields[6];
  &send;

  $name = "hfc$per\_hour";
  $value = @fields[7];
  &send;

  $name = "hfc$per\_min";
  $value = @fields[8];
  &send;

  $name = "hfc$per\_wday";
  $value = @fields[9];
#  &send;

  $name = "hfc$per\_wday_sh";
  $value = @fields[10];
#  &send;

  $name = "hfc$per\_tt";
  if (!$metric) {$value = @fields[11]*1.8+32} else {$value = @fields[11];}
  &send;

  $name = "hfc$per\_tt_fl";
  if (!$metric) {$value = @fields[12]*1.8+32} else {$value = @fields[12];}
  &send;

  $name = "hfc$per\_hi";
  if (!$metric) {$value = @fields[13]*1.8+32} else {$value = @fields[13];}
  &send;

  $name = "hfc$per\_hu";
  $value = @fields[14];
  &send;

  $name = "hfc$per\_w_dirdes";
  $value = @fields[15];
#  &send;

  $name = "hfc$per\_w_dir";
  $value = @fields[16];
  &send;

  $name = "hfc$per\_w_sp";
  if (!$metric) {$value = @fields[17]*0.621} else {$value = @fields[17];}
  &send;

  $name = "hfc$per\_w_ch";
  if (!$metric) {$value = @fields[18]*1.8+32} else {$value = @fields[18]};
  &send;

  $name = "hfc$per\_pr";
  $value = @fields[19];
  &send;

  $name = "hfc$per\_dp";
  $value = @fields[20];
  &send;

  $name = "hfc$per\_sky";
  $value = @fields[21];
  &send;

  $name = "hfc$per\_sky\_des";
  $value = @fields[22];
#  &send;

  $name = "hfc$per\_uvi";
  $value = @fields[23];
  &send;

  $name = "hfc$per\_prec";
  if (!$metric) {$value = @fields[24]*0.0393700787} else {$value = @fields[24];}
  &send;

  $name = "hfc$per\_snow";
  if (!$metric) {$value = @fields[25]*0.393700787} else {$value = @fields[25];}
  &send;

  $name = "hfc$per\_pop";
  $value = @fields[26];
  &send;

  $name = "hfc$per\_we_code";
  $value = @fields[27];
  &send;

  $name = "hfc$per\_we_icon";
  $value = @fields[28];
#  &send;


  $name = "hfc$per\_we_des";
  $value = @fields[29];
#  &send;

}

exit;

#
# Subroutines
#

# error Message
sub error {
  $logmessage = "ERROR: $errormessage\n";
  print "\n$logmessage\n";
  exit;
}  

sub send {
  # show value
  print "$name\@$value\n";
  return();
}

exit;
